{% extends "intern/base_site.html" %}
{% block extrahead %}
<script type="text/javascript" src="http://s.hdimg.net/jquery/jquery-1.3.1.min.js"></script>
<script src="http://s.hdimg.net/jquery/jquery.tagcloud.js" type="text/javascript"></script>
<style type="text/css">
body {
    background-color: #ffffff;
    font-family: "Lucida Grande","DejaVu Sans","Bitstream Vera Sans",Verdana,Arial,sans-serif; 
    color: #333333;
	font-size:12 px;
    text-align:left;
}
#tagcloud a { text-decoration: none; }
#tagcloud a:hover { text-decoration: underline; }
#tagcloud {font: 50%, Arial, "MS Trebuchet", sans-serif; text-align:justify;width:758px;}
</style>
{% endblock %}
{% block breadcrumbs %}<div class="breadcrumbs"><a href="/">Homepage</a>
&rsaquo;  {{ title }}</div>{% endblock %}
{% block content %}  
<br><br>
    <div id="imagematrix">
    {% for line in lines %}
        {% for col in line %}
{{ col }}{% endfor %}<br />
{% endfor %}
    </div>
    
      <div id="tagcloud">
{% for tag, count in tags %}
<a href="tag/{{ tag }}/" rel="{{ count }}">{{ tag }}</a>{% endfor %}
      </div>

<script type="text/javascript" charset="utf-8">
 $(document).ready(function(){
   $("#tagcloud a").tagcloud({
     size: {
       start: 8,
       end: 18,
       unit: "pt"
     }, 
     color: {
       start: "#aaa", 
       end: "#F52"
     }
   });
   $('#tagForm').submit(function() {
   return false;
 })
})
</script>

{% endblock content %}